package org.softwarepipelines.config;

import org.jaxen.JaxenException;
import org.jaxen.XPath;

import org.jaxen.dom.DOMXPath;

import org.softwarepipelines.config.exception.ConfigException;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import java.io.InputStream;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;


/**
 * @author Andy Grove
 */
public class DynamicConfigImpl implements DynamicConfig {
    private Document doc;
    private Map<String, XPath> xpathCache = new HashMap<String, XPath>();

    public DynamicConfigImpl(Document doc) {
        this.doc = doc;
    }

    public DynamicConfigImpl(InputStream is) throws ConfigException {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(false);

            DocumentBuilder builder = factory.newDocumentBuilder();
            doc = builder.parse(is);
        } catch (Exception e) {
            throw new ConfigException(e);
        }
    }

    public String getString(String path) throws ConfigException {
        try {
            XPath xpath = xpathCache.get(path);

            if (xpath == null) {
                xpath = new DOMXPath(path);
                xpathCache.put(path, xpath);
            }

            Node node = (Node) xpath.selectSingleNode(doc.getDocumentElement());

            if (node == null) {
                return null;
            }

            if (node instanceof Attr) {
                return ((Attr) node).getValue();
            }

            return node.toString();
        } catch (JaxenException e) {
            throw new ConfigException(e);
        }
    }

    public List<Element> getElements(String path) throws ConfigException {
        try {
            XPath xpath = xpathCache.get(path);

            if (xpath == null) {
                xpath = new DOMXPath(path);
                xpathCache.put(path, xpath);
            }

            List<?> nodes = xpath.selectNodes(doc.getDocumentElement());

            List<Element> ret = new ArrayList<Element>(nodes.size());

            for (Object obj : nodes) {
                if (obj instanceof Element) {
                    ret.add((Element) obj);
                }
            }

            return ret;
        } catch (JaxenException e) {
            throw new ConfigException(e);
        }
    }
}
