package org.softwarepipelines.config.dest;

import java.util.HashMap;

import org.softwarepipelines.config.PipelinesMasterConfig;


/**
 * DestinationMap instances contain a map of source 'service:distributor' name keys to DestinationInfo (distributor/connector name pair) instances.
 * DestinationMap instances also contain a reference to the PipelinesMasterConfig for this application context.
 * 
 * DestinationMaps are configured as singletons in Spring, and used to map source service instances to destination service instances
 * so that service invokes may be chained within the framework.  Services that wish to invoke another service in the framework need only
 * load the DestinationMap from the application context and use the PipelinesHelper.getConnectorFromDestMap
 * 
 * @author cholm
 *
 */
public class DestinationMap {
	
	final public static String DEST_MAP_BEAN_NAME = "destinationMapBean";
	
	// Pipelines Master Config
	public PipelinesMasterConfig masterConfig;
	
	// destination map
	public HashMap<String, DestinationInfo> destinationMap;
	
	public HashMap<String, DestinationInfo> getDestinationMap() {
		return destinationMap;
	}

	public void setDestinationMap(HashMap<String, DestinationInfo> destinationMap) {
		this.destinationMap = destinationMap;
	}

	public PipelinesMasterConfig getMasterConfig() {
		return masterConfig;
	}

	public void setMasterConfig(PipelinesMasterConfig masterConfig) {
		this.masterConfig = masterConfig;
	}
}
