package org.softwarepipelines.config.message;

import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.config.util.ConfigHelper;
import org.w3c.dom.Element;

public class MessageConfig {
	
	/* Type */
	protected Class<?> type;
	
	/* Pipeline Key */
	protected String pipelineKey;
	
	public MessageConfig() {
	}
	
	public void load(final Element message) throws ConfigException {
		// load the message type (fully qualified class name)
		String className = null;
		try {
			className = ConfigHelper.loadReqAttribute(message, ConfigHelper.TYPE_ATTR);
			this.setType(Class.forName(className));
		} catch (ClassNotFoundException cnfe) {
			throw new ConfigException("Error loading Message config.  Not a valid message type: " + className);
		}
		
		// load the optional pipeline key
		if (message.hasAttribute(ConfigHelper.PIPELINE_KEY_ATTR)) {
			this.setPipelineKey(message.getAttribute(ConfigHelper.PIPELINE_KEY_ATTR));
		}
	}

	public Class<?> getType() {
		return type;
	}

	public void setType(Class<?> type) {
		this.type = type;
	}

	public String getPipelineKey() {
		return pipelineKey;
	}

	public void setPipelineKey(String pipelineKey) {
		this.pipelineKey = pipelineKey;
	}

	public boolean equals(Object obj) {
		if (!(obj instanceof MessageConfig)) {
			return false;
		}
		MessageConfig config = (MessageConfig) obj;
		
		// check the type (fully qualified class name)
		if (!ConfigHelper.checkStringMemsEqual(this.getType().getName(), config.getType().getName())) {
			return false;
		}
		
		// and check the pipeline key
		if (!ConfigHelper.checkStringMemsEqual(this.getPipelineKey(), config.getPipelineKey())) {
			return false;
		}
		
		return true;
	}
}
