package org.softwarepipelines.config.router;

import java.util.ArrayList;
import java.util.List;

import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.config.message.MessageConfig;
import org.softwarepipelines.config.service.PipelineServiceConfig;
import org.softwarepipelines.config.util.ConfigHelper;
import org.w3c.dom.Element;

public class PipelinesRouterConfig {
	
	// Route Method Valid Values
	final public static String ROUND_ROBIN_RM = "round-robin";
	final public static String CONTENT_BASED_RM = "content-based";
	final public static String CUSTOM_RM = "custom";
	public static List<String> ROUTE_METHOD_VALID_VALUES;
	static {
		ROUTE_METHOD_VALID_VALUES = new ArrayList<String>();
		ROUTE_METHOD_VALID_VALUES.add(ROUND_ROBIN_RM);
		ROUTE_METHOD_VALID_VALUES.add(CONTENT_BASED_RM);
		ROUTE_METHOD_VALID_VALUES.add(CUSTOM_RM);
	}
	
	/* Route Method */
	//protected String routeMethod;
	
	/* Service Config */
	protected PipelineServiceConfig serviceConfig;
	
	/* Message Config */
	protected MessageConfig messageConfig;
	
	/* Class Name */
	protected String className;
	
	/* Enforce Sequence? */
	protected boolean enforceSequence;
	
	public PipelinesRouterConfig() {
	}
	
	public void load(final Element router) throws ConfigException {
		// load the route method
		//String rm = ConfigHelper.loadReqAttribute(router, ConfigHelper.ROUTE_METHOD_ATTR);
		
		// validate the route method is in the valid values list, and then set it
		//if (!ROUTE_METHOD_VALID_VALUES.contains(rm)) {
		//	throw new ConfigException("Invalid Message route-method: " + routeMethod);
		//}
		//this.setRouteMethod(rm);
		
		// load the class name
		this.setClassName(ConfigHelper.loadReqAttribute(router, ConfigHelper.CLASS_NAME_ATTR));
		
		// load the enforce sequence boolean
		this.setEnforceSequence(Boolean.parseBoolean(ConfigHelper.loadReqAttribute(router, ConfigHelper.ENFORCE_SEQUENCE_ATTR)));
	}

	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

	public boolean isEnforceSequence() {
		return enforceSequence;
	}

	public void setEnforceSequence(boolean enforceSequence) {
		this.enforceSequence = enforceSequence;
	}

	public boolean equals(Object obj) {
		if (!(obj instanceof PipelinesRouterConfig)) {
			return false;
		}
		PipelinesRouterConfig config = (PipelinesRouterConfig) obj;
		
		// check the route method
		//if (!ConfigHelper.checkStringMemsEqual(this.getRouteMethod(), config.getRouteMethod())) {
		//	return false;
		//}
		
		// check the class name
		if (!ConfigHelper.checkStringMemsEqual(this.getClassName(), config.getClassName())) {
			return false;
		}
		
		// and check if both enforce sequence
		if (this.isEnforceSequence() != config.isEnforceSequence()) {
			return false;
		}
		
		return true;
	}

	public MessageConfig getMessageConfig() {
		return messageConfig;
	}

	public void setMessageConfig(MessageConfig messageConfig) {
		this.messageConfig = messageConfig;
	}

	public PipelineServiceConfig getServiceConfig() {
		return serviceConfig;
	}

	public void setServiceConfig(PipelineServiceConfig serviceConfig) {
		this.serviceConfig = serviceConfig;
	}
}
