package org.softwarepipelines.examples.hello;

import org.apache.log4j.Logger;

import org.softwarepipelines.config.service.PipelineServiceConfig;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.service.PipelineServiceBase;

public class HelloPipelinesResponseService extends PipelineServiceBase {
    private static final Logger LOG = Logger.getLogger(HelloPipelinesResponseService.class);
    private static final int SLEEP_TIME = 50;
    private static final int SLEEP_CYCLES = 10;

    /**
     * Default constructor.
     */
    public HelloPipelinesResponseService() {
    }
    /**
     * Constructor used by the framework to pass in the 
     * PipelineServiceConfig instance.
     */
    public HelloPipelinesResponseService(final PipelineServiceConfig config) {
        super(config);
    }
    /**
     * Write the "Hello" message contents out to the log.
     */
    public synchronized void processMessage(AbstractMessage msg) throws PipelineException {
	    /* Cast the HelloMessage.*/
   		HelloMessage helloMessage = (HelloMessage)msg;
    	
    	/* Consume some CPU.*/ 
    	eatCpu();
    	
    	/* Write the message the to log.*/
    	LOG.debug("Hello " + helloMessage.getName() + " from SoftwarePipelines, in " + SLEEP_TIME + " ms!");
    	
    	/* Send the response. */
		HelloMessage toRet = new HelloMessage();
		toRet.setName("Software Pipelines");
		sendResultMessage(toRet);
    }

	/**
     * Consume some CPU time to simulate processor work.
     */
    private void eatCpu() {
        for (int i = 0; i < SLEEP_CYCLES; i++) {
            try {
                Thread.sleep(SLEEP_TIME/SLEEP_CYCLES);
            } catch (InterruptedException ie) {
                LOG.warn("Interrupted while sleeping for " + SLEEP_TIME + " ms to eat cpu", ie);
            }
        }
    }
}
