package org.softwarepipelines.examples.hello.ch14;

import java.io.File;
import java.io.IOException;

import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.util.PipelinesHelper;

/**
 * This client is the client for the HelloPipelinesService. It invokes the service 
 * via the local connector of the distributor.
 */
public class HelloPipelinesClient {

	/**
	 * Usage: HelloPipelinesClient [pipelinesConfig filename] [distributorName]
	 * @param args
	 * @throws IOException
	 * @throws ConfigException
	 * @throws PipelineException
	 */
	public static void main(String[] args) throws IOException, ConfigException, PipelineException{
		
		/* Get the parameters.*/
		if(args.length != 3) {
			throw new PipelineException("Usage: HelloPipelinesClient [pipelinesConfig filename] [distributorName] [connectorName]");
		}
		String pipelinesConfigFilename = args[0];
		String distributorName = args[1];
		String connectorName = args[2];
		
		/* Load the Pipelines Configuration.*/
		PipelinesMasterConfig masterConfig = new PipelinesMasterConfig(new File(pipelinesConfigFilename));
		
		/* Get a reference to the distributor by name, and start it.*/
		PipelinesDistributorBase distributor = PipelinesHelper.getDistributorByName(distributorName, masterConfig);
		distributor.start();
		
		/* Get a reference to the connector.*/
		PipelinesConnector connector = PipelinesHelper.getConnectorByName(distributorName, connectorName, masterConfig);
		
		/* Build the HelloMessage instance, including 'serviceName' and 'name'.*/
		HelloMessage msg = new HelloMessage();
		msg.setServiceName("helloService");
		msg.setName("Cory");
		
		/* Invoke the service on the distributor, via the connector.*/
		connector.send(msg);
		
		/* Cleanup by shutting down the connector and the distributor.*/
		connector.shutdown();
		distributor.shutdown();
	}

}
