package org.softwarepipelines.examples.hello.ch16;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.router.PipelinesRouterConfig;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.pipeline.Pipeline;
import org.softwarepipelines.framework.pipeline.PipelineImpl;
import org.softwarepipelines.framework.router.PipelinesRouterBase;

public class CustomContentBasedRouterImpl extends PipelinesRouterBase {
	private final static Logger LOG = Logger.getLogger( CustomContentBasedRouterImpl.class );
	
	public CustomContentBasedRouterImpl(){
	}
	
	/**
	 * Constructor to set pipelines configuration.
	 * @param config
	 * @param dist
	 * @throws PipelineException
	 */
	public CustomContentBasedRouterImpl(final PipelinesRouterConfig config, final PipelinesDistributorBase dist) throws PipelineException {
		super(config, dist);
	}

	/**
	 * Route the service request to the appropriate request. In this case,
	 * parse the first/last name values, and route based on the last name. 
	 */
	public void route(final AbstractMessage msg) throws PipelineException {
		/* Get the pipeline key value, the full name.*/ 
		String keyVal = (String) getPipelineKey(msg);
		
		/* Parse out the last name.*/
		String lastName = keyVal.substring(keyVal.indexOf(' ') + 1, keyVal.length());
		LOG.debug("Parsed last name '" + lastName + "' from full name '" + keyVal + "'");
		
		/* Match the last name to a pipeline.*/
		PipelineImpl plImpl;
		for (Pipeline pipeline : this.getPipelines()) {
			plImpl = (PipelineImpl) pipeline;
			if (plImpl.getConfig().getMatch().matcher(lastName).matches()) {
				LOG.debug("Invoking service for message with last name " + lastName + " in pipeline " + plImpl.getConfig().getName() + 
						" with match pattern " + plImpl.getConfig().getMatch().pattern());
				plImpl.invokeService(msg);
				return;
			}
		}
		
		/* If we get to this point, no matching pipline was found, so throw and error.*/ 
		throw new PipelineException("No pipeline regexp matched last name " + lastName + 
				"!\nPipelines: " + config.getServiceConfig().getPipelines());
	}
}
