package org.softwarepipelines.examples.hello.ch16;

import java.io.File;
import java.io.IOException;
import java.util.Random;

import org.jperf.PerfTest;
import org.jperf.PerfTestRunner;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.util.PipelinesHelper;

public class HelloPipelinesContentBasedPerfTest implements PerfTest {
	
	/* Pipelines Master Config */
	private static PipelinesMasterConfig masterConfig;

	/* Singleton static distributor instance used by all the threads.*/
	private static PipelinesDistributorBase distributor;
	
	/* Connector instance which each thread will instantiate separately.*/
	private PipelinesConnector connector;
	private static String connectorName;
	
	/* Define an array of names throughout the alphabet.*/
	String[] names = {"Abbey", "Barbara", "Colin", 
			"Devyn", "Edward", "Fred", "George", 
			"Harry", "Isabel", "John", "Karl", "Larry", 
			"Monty", "Nancy", "Oprah", "Patrick", 
			"Quinn", "Roger", "Sam", "Tyler", 
			"Ursula", "Victor", "Wayne", 
			"Xavier", "Yolanda", "Zeb"}; 
	
	/* Random number generator.*/
	Random random = new Random();
	
	/** 
	 * Set up the test.
	 */
	public void setUp() throws Exception {
		/* Check out a local connector instance.*/
		connector = PipelinesHelper.getConnectorByName(distributor.getConfig().getName(), connectorName, masterConfig);
	}

	/**
	 * Tear down the test, performing any cleanup.
	 */
	public void tearDown() throws Exception {
		/* Shut the connector down.*/
		connector.shutdown();
	}

	/**
	 * Run the service test.
	 */
	public void test() throws Exception {
		/* Build the HelloMessage instance, including 'serviceName' and 'name'.*/
		HelloMessage msg = new HelloMessage();
		msg.setServiceName("helloService");
		
		/* Set the name to a random value from the array.*/
		msg.setName(names[random.nextInt(26)]);
		
		/* Invoke the service on the distributor, via the connector.*/
		connector.send(msg);
	}

	/**
	 * Main method for starting the test.
	 * @param args
	 * @throws IOException
	 * @throws ConfigException
	 * @throws PipelineException
	 */
	public static void main(String[] args) throws IOException, ConfigException, PipelineException {
		String pipelinesConfigFilename, distributorName;
		
		/* Get the parameters.*/
		if(args.length != 3) {
			throw new PipelineException("Usage: HelloPipelinesClient [pipelinesConfig filename] [distributorName] [connectorName]");
		}
		
		pipelinesConfigFilename = args[0];
		distributorName = args[1];
		connectorName = args[2];
				
		/* Load the Pipelines Configuration.*/
		masterConfig = new PipelinesMasterConfig(new File(pipelinesConfigFilename));
		
		/* Get a reference to the distributor by name.*/
		distributor = PipelinesHelper.getDistributorByName(distributorName, masterConfig);
		distributor.start();
		
		/* Set up the PerfTest runner for 6 threads, 4 seconds between threads.*/
		PerfTestRunner r = new PerfTestRunner();
        r.setMaxClient(6);
        r.setTestPeriod(4000);
        
        /* Specify the results output file.*/
        r.setResultFile(new File("results/test/HelloPipelinesContentBasedPerfTest-results.xml"));
        
        /* Run the test.*/
        r.run(HelloPipelinesContentBasedPerfTest.class);
        
        /* Shutdown the distributor.*/
        distributor.shutdown();
	}

}
