package org.softwarepipelines.examples.hello.ch17;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.util.PipelinesHelper;

/**
 * This client is the client for the HelloPipelinesService. It invokes the service 
 * via the local connector of the distributor.
 */
public class HelloPipelinesResponseClient {
	final private static Logger LOG = Logger.getLogger(HelloPipelinesResponseClient.class);

	/**
	 * Usage: HelloPipelinesClient [pipelinesConfig filename] [distributorName]
	 * @param args
	 * @throws IOException
	 * @throws ConfigException
	 * @throws PipelineException
	 */
	public static void main(String[] args) throws IOException, ConfigException, PipelineException{
		
		/* Get the parameters.*/
		if(args.length != 3) {
			throw new PipelineException("Usage: HelloPipelinesClient [pipelinesConfig filename] [distributorName] [connectorName]");
		}
		String pipelinesConfigFilename = args[0];
		String distributorName = args[1];
		String connectorName = args[2];
		
		/* Load the Pipelines Configuration.*/
		PipelinesMasterConfig masterConfig = new PipelinesMasterConfig(
				new File(pipelinesConfigFilename));
		
		/* Get the remote destination distributor and start it */
		PipelinesDistributorBase distributor = PipelinesHelper.getDistributorByName(distributorName, masterConfig);
		distributor.start();
		
		/* Check out a connector instance from the client broker.*/
		PipelinesConnector connector = PipelinesHelper.getConnectorByName(distributorName, connectorName, masterConfig);
		
		/* Build the HelloMessage instance, including 'serviceName' and 'name'.*/
		HelloMessage msg = new HelloMessage();
		msg.setServiceName("helloService");
		msg.setName("Cory");
		
		/* Invoke the service on the distributor, via the connector.*/
		LOG.info("Sending message: " + msg);
		AbstractMessage resp = connector.sendAndReceive(msg);
		
		/* Cast the response to a HelloMessage */
		HelloMessage helloResp = (HelloMessage) resp;
		LOG.info("Received response from: " + helloResp.getName());
		
		/* Cleanup by shutting down the singleton distributor and all message brokers.*/
		connector.shutdown();
		distributor.shutdown();
	}

}
