package org.softwarepipelines.examples.hello.ch17;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.jperf.PerfTest;
import org.jperf.PerfTestRunner;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.util.PipelinesHelper;

public class HelloPipelinesResponsePerfTest implements PerfTest {
	final private static Logger LOG = Logger.getLogger(HelloPipelinesResponsePerfTest.class);
	
	/* Master Pipelines Config */
	private static PipelinesMasterConfig masterConfig;

	/* Message Brokers */
	private static PipelinesDistributorBase distributor;
	
	/* Connector instance which each thread will instantiate separately.*/
	private PipelinesConnector connector;
	private static String connectorName;
	
	/** 
	 * Set up the test.
	 */
	public void setUp() throws Exception {
		/* Check out a socket connector instance.*/
		connector = PipelinesHelper.getConnectorByName(distributor.getConfig().getName(), connectorName, masterConfig);
	}

	/**
	 * Tear down the test, performing any cleanup.
	 */
	public void tearDown() throws Exception {
		// shut down connector
		connector.shutdown();
	}

	/**
	 * Run the service test.
	 */
	public void test() throws Exception {
		/* Build the HelloMessage instance, including 'serviceName' and 'name'.*/
		HelloMessage msg = new HelloMessage();
		msg.setServiceName("helloService");
		msg.setName("Cory");
		LOG.debug("Sending message: " + msg);
		
		/* Invoke the service on the distributor, via the connector.*/
		HelloMessage resp = (HelloMessage)connector.sendAndReceive(msg);
		LOG.debug("Received response: " + resp);
	}

	/**
	 * Main method for starting the test.
	 * @param args
	 * @throws IOException
	 * @throws ConfigException
	 * @throws PipelineException
	 */
	public static void main(String[] args) throws IOException, ConfigException, PipelineException {
		String pipelinesConfigFilename, distributorName;
		
		/* Get the parameters.*/
		if(args.length != 3) {
			throw new PipelineException("Usage: HelloPipelinesClient [pipelinesConfig filename] [distributorName]");
		}
		pipelinesConfigFilename = args[0];
		distributorName = args[1];
		connectorName = args[2];
				
		/* Load the Pipelines Configuration.*/
		masterConfig = new PipelinesMasterConfig(new File(pipelinesConfigFilename));
		
		/* Setup the destination distributor */
		distributor = PipelinesHelper.getDistributorByName(distributorName, masterConfig);
		distributor.start();
		
		/* Set up the PerfTest runner for 6 threads, 4 seconds between threads.*/
		PerfTestRunner r = new PerfTestRunner();
        r.setMaxClient(6);
        r.setTestPeriod(4000);
        
        /* Specify the results output file.*/
        r.setResultFile(new File("results/test/HelloPipelinesResponsePerfTest-results.xml"));
        
        /* Run the test.*/
        r.run(HelloPipelinesResponsePerfTest.class);
        
        /* Shutdown the brokers and executor threads.*/
        distributor.shutdown();

	}

}
