package org.softwarepipelines.examples.hello.ch21;

import org.apache.log4j.Logger;
import org.softwarepipelines.framework.util.Instrumentor;
import org.softwarepipelines.framework.util.Counter;

/**
 * A simple example of using the Instrumentor class. 
 *
 */
public class HelloPipelinesInstrumentorExample {

    private static final Logger LOG = Logger.getLogger(HelloPipelinesInstrumentorExample.class);
    private static final int SLEEP_TIME = 50;
    private static final int SLEEP_CYCLES = 10;
    private static final int LOG_INTERVAL = 2000;

    /* Get the Instrumentor instance.*/
    private final Instrumentor instr = Instrumentor.getInstrumentor(HelloPipelinesInstrumentorExample.class);
    
    /* Get the Counter instance.*/
    private final Counter helloCounter = instr.getCounter("helloCounter");
    
    /**
     * Default constructor.
     */
    public HelloPipelinesInstrumentorExample() {
    	/* Set the log interval.*/
    	Instrumentor.setLogInterval(LOG_INTERVAL);
    }

    /**
     * Write the "Hello" message contents out to the log.
     */
    public synchronized void sayHello(String name) {
    	
    	/* Instrument the method.*/
    	helloCounter.next();
    	
    	/* Consume some CPU.*/ 
    	eatCpu();
    	
    	/* Write the message the to log.*/
    	LOG.debug("Hello " + name + " from SoftwarePipelines, in " + SLEEP_TIME + " ms!");
    }

	/**
     * Consume some CPU time to simulate processor work.
     */
    private void eatCpu() {
        for (int i = 0; i < SLEEP_CYCLES; i++) {
            try {
                Thread.sleep(SLEEP_TIME/SLEEP_CYCLES);
            } catch (InterruptedException ie) {
                LOG.warn("Interrupted while sleeping for " + SLEEP_TIME + " ms to eat cpu", ie);
            }
        }
    }

	/**
	 * Set up and run the example.
	 */
	public static void main(String[] args) {
		
		HelloPipelinesInstrumentorExample helloExample = new HelloPipelinesInstrumentorExample();

		/* Say hello in a loop.*/
		for(int i=0;i<100;i++) {
	    	helloExample.sayHello("Cory");
	    }

	}

}
