package org.softwarepipelines.examples.hello.id;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.service.PipelineServiceConfig;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.service.PipelineServiceBase;

public class HelloIdPipelinesService extends PipelineServiceBase {
	private static final Logger LOG = Logger.getLogger(HelloIdPipelinesService.class);
    private static final int SLEEP_TIME = 50;
    private static final int SLEEP_CYCLES = 10;

	public HelloIdPipelinesService() {
	}

	public HelloIdPipelinesService(PipelineServiceConfig config) {
		super(config);
	}
	
	public synchronized void processMessage(AbstractMessage msg) throws PipelineException {
    	// eat cpu to simulate work
    	eatCpu();
    	
    	// log the hello message
    	LOG.debug("Hello ID " + ((HelloIdMessage)msg).getId() + " from SoftwarePipelines, in " + SLEEP_TIME + " ms!");
    	
    	// and send message if we are a return service
    	if (!getConfig().isOneWay()) {
    		HelloIdMessage toRet = new HelloIdMessage();
    		toRet.setConnectorRef(msg.getConnectorRef());
    		toRet.setServiceName(msg.getServiceName());
    		toRet.setSenderHost(msg.getSenderHost());
    		toRet.setSenderPort(msg.getSenderPort());
    		toRet.setId(-1);
    		sendResultMessage(toRet);
    	}
    }

    private void eatCpu() {
        // eat CPU, sleeping for a total of SLEEP_TIME ms
        for (int i = 0; i < SLEEP_CYCLES; i++) {
            try {
                Thread.sleep(SLEEP_TIME/SLEEP_CYCLES);
            } catch (InterruptedException ie) {
                LOG.warn("Interrupted while sleeping for " + SLEEP_TIME + " ms to eat cpu", ie);
            }
        }
    }

}
