package org.softwarepipelines.examples.pbcor;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.softwarepipelines.framework.message.AbstractMessage;

public class AccountTransMessage extends AbstractMessage {
	
	/* Valid transaction flag; default to true */
	protected boolean valid = true;
	
	/* Field for detailing success, validation or authorization errors.*/
	protected String msg;

	/* Customer Account number */
	public long acctNum;
	
	/* Branch ID */
	public int branchId;
	
	/* PIN number */
	protected int pin;
	
	/**
	 * Default constructor
	 */
	public AccountTransMessage() {
	}
	
	public long getAcctNum() {
		return acctNum;
	}

	public void setAcctNum(long acctNum) {
		this.acctNum = acctNum;
	}

	public int getPin() {
		return pin;
	}

	public void setPin(int pin) {
		this.pin = pin;
	}

	public int getBranchId() {
		return branchId;
	}

	public void setBranchId(int branchId) {
		this.branchId = branchId;
	}

	public boolean isValid() {
		return valid;
	}

	public void setValid(boolean valid) {
		this.valid = valid;
	}

	public String getMsg() {
		return msg;
	}

	public void setMsg(String msg) {
		this.msg = msg;
	}
	
	/**
	 * Implement writeExternal for efficient serialization
	 * of the message. This is used for the socket 
	 * connector type, and only serializes the specific
	 * fields required.
	 */
	public void writeExternal(ObjectOutput out) throws IOException {
		super.writeExternal(out);
		
		/* Write the valid flag.*/
		out.writeBoolean(this.isValid());
		
		/* If invalid, write the message.*/
		if (!this.isValid()) {
			out.writeUTF(this.getMsg());
		}
		
		/* Write the account number.*/
		out.writeLong(this.getAcctNum());
		
		/* Write the branch id.*/
		out.writeInt(this.getBranchId());
		
		/* Write the pin.*/
		out.writeInt(this.getPin());
	}
	
	/**
	 * Implement readExternal for efficient de-serialization
	 * of the message. This is used for the socket 
	 * connector type, and only reads  the specific
	 * fields required.
	 */
	public void readExternal(ObjectInput in) throws IOException {
		super.readExternal(in);
		
		/* Read the valid flag.*/
		this.setValid(in.readBoolean());
		
		/* If invalid, read the message.*/
		if (!this.isValid()) {
			this.setMsg(in.readUTF());
		}
		
		/* Read in the account number.*/
		this.setAcctNum(in.readLong());
		
		/* Read the branch id.*/
		this.setBranchId(in.readInt());
		
		/* Read the pin.*/
		this.setPin(in.readInt());
	}
	
	/**
	 * Output the message as a string.
	 */
	public String toString() {
		return super.toString() + "\n\tAcct Num: " + this.getAcctNum() +
			"\n\tBranch ID: " + this.getBranchId() +
			"\n\tPIN: " + this.getPin() +
			"\n\tValid?: " + this.isValid() +
			"\n\tMessage: " + this.getMsg();
	}
}
