package org.softwarepipelines.examples.pbcor;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.service.PipelineServiceConfig;
import org.softwarepipelines.examples.pbcor.acctdb.Account;
import org.softwarepipelines.examples.pbcor.acctdb.AccountDB;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.service.PipelineServiceBase;
import org.softwarepipelines.framework.util.PipelinesHelper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * PBCOR AuthorizeTransactionService
 */
public class AuthorizeTransactionService extends PipelineServiceBase {
	private static final Logger LOG = Logger.getLogger(AuthorizeTransactionService.class);
	
	/* Connector through which we will invoke validate transaction */
	protected PipelinesConnector connector;
	
	/* Target Service; the next service in the invoke chain */
	protected String targetService;
	
	/* Account DB */
	private AccountDB acctDB;
	
	public AuthorizeTransactionService(PipelineServiceConfig config) throws PipelineException {
		super(config);

		/* Get the Spring application context */
		ApplicationContext ac = new ClassPathXmlApplicationContext(
				"pbcorAppContext.xml");
		
		/* Set the target service */
		targetService = "validateService";

		/* Load the connector for the validationService 
		 * from the destination map configured in the application context.*/
		LOG.debug("Getting connector with base key: " + getDestMapKey() + " for target service " + targetService);
		connector = PipelinesHelper.getConnectorFromDestMap(ac, getDestMapKey(), targetService);

		/* Get the AccountDB */
		acctDB = (AccountDB) ac.getBean("accountDB");
	}

	public void processMessage(AbstractMessage msg) throws PipelineException {
		/* Validate that we received an AccountTransMessage */
		if (!(msg instanceof AccountTransMessage)) {
			throw new PipelineException("AuthorizeTransaction service requires a message of type " + AccountTransMessage.class.getName());
		}
		
		/* And cast the incoming message */
		AccountTransMessage acctMsg = (AccountTransMessage) msg;
		
		/* Get the account with the given account number from the AccountDB */
		//Account acct = acctDB.getAccountByNumber(acctMsg.getAcctNum());
		
		/* Check that we have a valid account */ 
		//if (acct == null) {
		//	acctMsg.setValid(false);
		//	acctMsg.setMsg("Invalid Account Number!");
		//	sendResultMessage(acctMsg);
		//}
		
		/* And that the pin numbers match; if so, invoke validate transaction */
		//if (acct.getPin() == acctMsg.getPin()) {
			acctMsg.setServiceName(targetService);
			connector.send(acctMsg);
		//}
		/* If not, set the invalid flag and return */
		//else {
		//	acctMsg.setValid(false);
		//	acctMsg.setMsg("Invalid PIN");
		//	sendResultMessage(acctMsg);
		//}
	}
	
	/**
	 * Finally, implement our shutdown logic.
	 */
	public void shutdown() throws PipelineException {
		super.shutdown();
		connector.shutdown();
		acctDB.shutdown();
	}
}
