package org.softwarepipelines.examples.pbcor;

import org.softwarepipelines.config.service.PipelineServiceConfig;
import org.softwarepipelines.examples.pbcor.acctdb.Account;
import org.softwarepipelines.examples.pbcor.acctdb.AccountDB;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.service.PipelineServiceBase;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * Service for Debit of an account.
 */
public class DebitAcctService extends PipelineServiceBase {
	
	/* Account Database Communication Service */
	private AccountDB acctDB;
	
	/**
	 * Default constructor.
	 */
	public DebitAcctService() {
	}

	/**
	 * Required service constructor.
	 * @param config
	 */
	public DebitAcctService(PipelineServiceConfig config) {
		super(config);
		
		/* Load the account db from the application context */
		ApplicationContext ac = new ClassPathXmlApplicationContext("pbcorAppContext.xml");
		
		/* Load the AccountDB bean */
		acctDB = (AccountDB) ac.getBean("accountDB");
	}

	/**
	 * Process the message.
	 */
	public void processMessage(AbstractMessage msg) throws PipelineException {
		/* Validate that we have a DebitAccountMessage and cast it.*/
		if (!(msg instanceof DebitAcctMessage)) {
			throw new PipelineException(DebitAcctService.class.getName() + " requires message type " + DebitAcctMessage.class.getName());
		}
		DebitAcctMessage debitMsg = (DebitAcctMessage) msg;
		
		/* Get the account from the accout db.*/
		Account acct = acctDB.getAccountByNumber(debitMsg.getAcctNum());
		float currBalance = acct.getBalance();
		synchronized (acct) {
			acct.setBalance(currBalance - debitMsg.getAmount());
		}
		
		/* And send a successful result message.*/
		debitMsg.setMsg("Successfully debited $" + debitMsg.getAmount() + " from account #" + debitMsg.getAcctNum());
		debitMsg.setServiceName("authorizeService");
		sendResultMessage(debitMsg);
	}
	
	/**
	 * Finally, implement our shutdown logic
	 */
	public void shutdown() throws PipelineException {
		super.shutdown();
		acctDB.shutdown();
	}

}
