package org.softwarepipelines.examples.pbcor.acctdb;

import java.util.HashMap;

import org.softwarepipelines.framework.exception.PipelineException;

public class AccountDBMemory implements AccountDB {
	
	// account number to account map
	protected static HashMap<Long, Account> acctNumToAcctMap;
	
	public AccountDBMemory() {
	}
	
	/**
	 * Initializes the MockDB.
	 */
	public void init() {
		// create the account map
		acctNumToAcctMap = new HashMap<Long, Account>();
		
		// build four accounts for now, one in each branch id pipeline
		Account acct;
		for (int i = 0;i < 4; i++) {
			acct = new Account();
			acct.setAcctNum(1000 + i);
			acct.setBranchId((i*10) + 1);
			acct.setBalance(100);
			acct.setPin(1000 + i);
			acctNumToAcctMap.put(acct.getAcctNum(), acct);
		}
		
	}
	
	public HashMap<Long, Account> getAcctNumToAcctMap() {
		return acctNumToAcctMap;
	}

	public void setAcctNumToAcctMap(HashMap<Long, Account> anAcctNumToAcctMap) {
		acctNumToAcctMap = anAcctNumToAcctMap;
	}

	/**
	 * Adds a mock account to the database.
	 * 
	 * @param acct the account to add to the database
	 */
	public void addAccountToDB(final Account acct) throws PipelineException {
		// check that we don't already have an account with that id
		if (acctNumToAcctMap.containsKey(acct.getAcctNum())) {
			throw new PipelineException("Database already contains an account with the account number: " + acct.getAcctNum() + 
					" at branch " + acct.getBranchId());
		}
		
		// and then add the account to the map
		acctNumToAcctMap.put(acct.getAcctNum(), acct);
	}
	
	/**
	 * Remove an account from the mock D
	 * @param acctNum
	 * @throws PipelineException
	 */
	public void removeAccounttFromDB(final long acctNum) throws PipelineException {
		if (!acctNumToAcctMap.containsKey(acctNum)) {
			throw new PipelineException("No account in DB with account number: " + acctNum);
		}
		acctNumToAcctMap.remove(acctNum);
	}
	
	/**
	 * @see AccountDB
	 */
	public synchronized Account getAccountByNumber(long acctNum) {
		
		/* Sleep for 2ms to simulate a real database write at 1000TPS.*/
		eatCpu(1);
		
		// check if the account map contains our key; if so, return the account
		if (acctNumToAcctMap.containsKey(acctNum)) {
			return acctNumToAcctMap.get(acctNum);
		}
		
		// return null if no such account exists
		return null;
	}
	
	/**
	 * @see AccountDB
	 */
	public synchronized void saveAccount(Account acct) {
		/* Sleep for 2ms to simulate a real database write at 500TPS.*/
		eatCpu(2);
		acctNumToAcctMap.put(acct.getAcctNum(), acct);
	}
	
	/**
	 * Sleep to simulate database access.
	 */
	private void eatCpu(int count) {
		try {Thread.sleep(count);} catch (Exception e){}
	}

	/**
	 * Shut down the AccountDB.
	 */
	public void shutdown() {
		setAcctNumToAcctMap(null);
	}
}
