package org.softwarepipelines.examples.pbcor.debitonly;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.dest.DestinationInfo;
import org.softwarepipelines.examples.pbcor.DebitAcctMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.util.PipelinesHelper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public class DebitAcctClient {
	final private static Logger LOG = Logger.getLogger(DebitAcctClient.class);
	
	public PipelinesMasterConfig configBean;
	public DestinationInfo authDestInfo, valDestInfo, debitDestInfo;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws PipelineException {
		ApplicationContext ac;
		DebitAcctClient testClient;
		PipelinesDistributorBase authDist, valDist, debitDist;
		
		// Load the application context
		ac = new ClassPathXmlApplicationContext("pbcorAppContext.xml");
		
		// and load the appropriate test client bean
		testClient = (DebitAcctClient) ac.getBean("testClient");
		
		// get the distributor for each of the three services
		authDist = PipelinesHelper.getDistributorByName(testClient.getAuthDestInfo().getDestinationDistributor(), testClient.getConfigBean());
		valDist = PipelinesHelper.getDistributorByName(testClient.getValDestInfo().getDestinationDistributor(), testClient.getConfigBean());
		debitDist = PipelinesHelper.getDistributorByName(testClient.getDebitDestInfo().getDestinationDistributor(), testClient.getConfigBean());
		
		// and start the distributors
		authDist.start();
		valDist.start();
		debitDist.start();
		
		// get the connector
		PipelinesConnector connector = PipelinesHelper.getConnectorByName(testClient.getAuthDestInfo().getDestinationDistributor(),
				testClient.getAuthDestInfo().getDestinationConnector(), testClient.getConfigBean());
		
		// build a debit account message
		DebitAcctMessage msg = new DebitAcctMessage();
		msg.setAcctNum(1001);
		msg.setAmount(50);
		msg.setBranchId(11);
		msg.setPin(1001);
		msg.setServiceName("authorizeService");
		
		
		// and invoke the debit account service
		LOG.info("Sending debit account message: " + msg);
		DebitAcctMessage resp = (DebitAcctMessage) connector.sendAndReceive(msg);
		LOG.info("Received response: " + resp);
		
		// finally,shut down the connector and the distributors
		connector.shutdown();
		authDist.shutdown();
		valDist.shutdown();
		debitDist.shutdown();
	}
	
	public PipelinesMasterConfig getConfigBean() {
		return configBean;
	}

	public void setConfigBean(PipelinesMasterConfig configBean) {
		this.configBean = configBean;
	}

	public DestinationInfo getAuthDestInfo() {
		return authDestInfo;
	}

	public void setAuthDestInfo(DestinationInfo authDestInfo) {
		this.authDestInfo = authDestInfo;
	}

	public DestinationInfo getValDestInfo() {
		return valDestInfo;
	}

	public void setValDestInfo(DestinationInfo valDestInfo) {
		this.valDestInfo = valDestInfo;
	}

	public DestinationInfo getDebitDestInfo() {
		return debitDestInfo;
	}

	public void setDebitDestInfo(DestinationInfo debitDestInfo) {
		this.debitDestInfo = debitDestInfo;
	}
}
