package org.softwarepipelines.examples.pbcor.ec2;

import java.io.File;
import java.io.IOException;
import java.util.Random;

import org.apache.log4j.Logger;
import org.jperf.PerfTest;
import org.jperf.PerfTestRunner;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.dest.DestinationInfo;
import org.softwarepipelines.examples.pbcor.DebitAcctMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.util.PipelinesHelper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public class DebitAcctPerfTest implements PerfTest {
	final private static Logger LOG = Logger.getLogger(DebitAcctPerfTest.class);
	
	public PipelinesMasterConfig configBean;
	public DestinationInfo authDestInfo, valDestInfo;
	
	public static String distributorName;
	public static String connectorName;
	public static PipelinesMasterConfig masterConfig;
	
	/* Connector */
	private PipelinesConnector connector;
	
	/* Random number generator.*/
	Random random = new Random();
	
	public void setUp() throws Exception {
		// get the connector
		connector = PipelinesHelper.getConnectorByName(distributorName, connectorName, masterConfig);
	}

	public void tearDown() throws Exception {
		// shutdown the connector
		connector.shutdown();

	}

	public void test() throws Exception {
		DebitAcctMessage msg, resp;
		
		// generate a random number between one and four to set our acct number, brach id and pin 
		int incr = random.nextInt(4);
		int acctIncr = random.nextInt(10);
		
		// build a debit account message
		msg = new DebitAcctMessage();
		msg.setAcctNum(1000*acctIncr + incr + 1);
		msg.setBranchId(10*incr + 1);
		msg.setPin(1000*acctIncr + incr + 1);
		msg.setServiceName("authorizeService");
		
		// set the amount to 0 so that we can do as many as possible
		msg.setAmount(0);
		
		LOG.debug("Sending message: " + msg);
		resp = (DebitAcctMessage) connector.sendAndReceive(msg);
		LOG.debug("Received response: " + resp);
	} 
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws PipelineException, IOException {
		ApplicationContext ac;
		DebitAcctPerfTest perfTest;
		PipelinesDistributorBase authDist, valDist;
		
		// Load the application context
		ac = new ClassPathXmlApplicationContext("pbcorAppContext.xml");
		
		// and load the appropriate test client bean
		perfTest = (DebitAcctPerfTest) ac.getBean("perfTest");
		
		// set the static master config
		masterConfig = perfTest.getConfigBean();
		distributorName = perfTest.getAuthDestInfo().getDestinationDistributor();
		connectorName = perfTest.getAuthDestInfo().getDestinationConnector();
		
		// get the distributor for each of the three services
		authDist = PipelinesHelper.getDistributorByName(perfTest.getAuthDestInfo().getDestinationDistributor(), perfTest.getConfigBean());
		valDist = PipelinesHelper.getDistributorByName(perfTest.getValDestInfo().getDestinationDistributor(), perfTest.getConfigBean());
		
		// and start the distributors
		authDist.start();
		valDist.start();
		
		/* Set up the PerfTest runner for 6 threads, 4 seconds between threads.*/
		PerfTestRunner r = new PerfTestRunner();
        r.setMaxClient(6);
        r.setTestPeriod(4000);
        
        /* Specify the results output file.*/
        r.setResultFile(new File("results/test/DebitAcctPerfTest-ec2-results.xml"));
        
        /* Run the test.*/
        r.run(DebitAcctPerfTest.class);
		
		// finally,shut down the distributors
		authDist.shutdown();
		valDist.shutdown();
	}
	
	public PipelinesMasterConfig getConfigBean() {
		return configBean;
	}

	public void setConfigBean(PipelinesMasterConfig configBean) {
		this.configBean = configBean;
	}

	public DestinationInfo getAuthDestInfo() {
		return authDestInfo;
	}

	public void setAuthDestInfo(DestinationInfo authDestInfo) {
		this.authDestInfo = authDestInfo;
	}

	public DestinationInfo getValDestInfo() {
		return valDestInfo;
	}

	public void setValDestInfo(DestinationInfo valDestInfo) {
		this.valDestInfo = valDestInfo;
	}	
}
