package org.softwarepipelines.examples.pbcor.full;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.dest.DestinationInfo;
import org.softwarepipelines.examples.pbcor.DebitAcctMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.util.PipelinesHelper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public class DebitAcctClient {
	final private static Logger LOG = Logger.getLogger(DebitAcctClient.class);
	
	public PipelinesMasterConfig configBean;
	public DestinationInfo authDestInfo, valDestInfo, val1DestInfo, val2DestInfo, val3DestInfo, val4DestInfo;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws PipelineException {
		ApplicationContext ac;
		DebitAcctClient testClient;
		PipelinesDistributorBase authDist, valDist, valDist1, valDist2, valDist3, valDist4;
		
		// Load the application context
		ac = new ClassPathXmlApplicationContext("pbcorAppContext.xml");
		
		// and load the appropriate test client bean
		testClient = (DebitAcctClient) ac.getBean("testClient");
		
		// get the distributor for each of the three services
		authDist = PipelinesHelper.getDistributorByName(testClient.getAuthDestInfo().getDestinationDistributor(), testClient.getConfigBean());
		valDist = PipelinesHelper.getDistributorByName(testClient.getValDestInfo().getDestinationDistributor(), testClient.getConfigBean());
		valDist1 = PipelinesHelper.getDistributorByName(testClient.getVal1DestInfo().getDestinationDistributor(), testClient.getConfigBean());
		valDist2 = PipelinesHelper.getDistributorByName(testClient.getVal2DestInfo().getDestinationDistributor(), testClient.getConfigBean());
		valDist3 = PipelinesHelper.getDistributorByName(testClient.getVal3DestInfo().getDestinationDistributor(), testClient.getConfigBean());
		valDist4 = PipelinesHelper.getDistributorByName(testClient.getVal4DestInfo().getDestinationDistributor(), testClient.getConfigBean());
		
		// and start the distributors
		authDist.start();
		valDist.start();
		valDist1.start();
		valDist2.start();
		valDist3.start();
		valDist4.start();
		
		// get the connector
		PipelinesConnector connector = PipelinesHelper.getConnectorByName(testClient.getAuthDestInfo().getDestinationDistributor(),
				testClient.getAuthDestInfo().getDestinationConnector(), testClient.getConfigBean());
		
		// build a debit account message
		DebitAcctMessage msg = new DebitAcctMessage();
		msg.setAcctNum(1001);
		msg.setAmount(50);
		msg.setBranchId(11);
		msg.setPin(1001);
		msg.setServiceName("authorizeService");
		
		
		// and invoke the debit account service
		LOG.info("Sending debit account message: " + msg);
		DebitAcctMessage resp = (DebitAcctMessage) connector.sendAndReceive(msg);
		LOG.info("Received response: " + resp);
		
		// finally,shut down the connector and the distributors
		connector.shutdown();
		authDist.shutdown();
		valDist.shutdown();
		valDist1.shutdown();
		valDist2.shutdown();
		valDist3.shutdown();
		valDist4.shutdown();
	}
	
	public PipelinesMasterConfig getConfigBean() {
		return configBean;
	}

	public void setConfigBean(PipelinesMasterConfig configBean) {
		this.configBean = configBean;
	}

	public DestinationInfo getAuthDestInfo() {
		return authDestInfo;
	}

	public void setAuthDestInfo(DestinationInfo authDestInfo) {
		this.authDestInfo = authDestInfo;
	}

	public DestinationInfo getValDestInfo() {
		return valDestInfo;
	}

	public void setValDestInfo(DestinationInfo valDestInfo) {
		this.valDestInfo = valDestInfo;
	}

	public DestinationInfo getVal1DestInfo() {
		return val1DestInfo;
	}

	public void setVal1DestInfo(DestinationInfo val1DestInfo) {
		this.val1DestInfo = val1DestInfo;
	}

	public DestinationInfo getVal2DestInfo() {
		return val2DestInfo;
	}

	public void setVal2DestInfo(DestinationInfo val2DestInfo) {
		this.val2DestInfo = val2DestInfo;
	}

	public DestinationInfo getVal3DestInfo() {
		return val3DestInfo;
	}

	public void setVal3DestInfo(DestinationInfo val3DestInfo) {
		this.val3DestInfo = val3DestInfo;
	}

	public DestinationInfo getVal4DestInfo() {
		return val4DestInfo;
	}

	public void setVal4DestInfo(DestinationInfo val4DestInfo) {
		this.val4DestInfo = val4DestInfo;
	}
}
