package org.softwarepipelines.examples.pbcor.full;

import com.dbshards.jdbc.ShardResolver;
import com.dbshards.jdbc.DbsConnection;
import com.dbshards.jdbc.DbsStatement;
import com.dbshards.jdbc.DefaultShardResolver;
import com.dbshards.core.config.ShardConfig;

import java.util.List;
import java.util.ArrayList;

import org.apache.log4j.Logger;

/**
 * Shard resolver for dbShards. This eventually won't be needed for such a simple case but for now it is.
 *
 * @author Andy Grove
 * @created Jul 15, 2008
 */
public class PbcorShardResolver extends DefaultShardResolver {

    private static final Logger logger = Logger.getLogger(PbcorShardResolver.class);

    private static final String SQL_SELECT_ACCOUNT = "SELECT account_no, branch_id, pin, balance FROM account WHERE account_no = ?";

    private static final String SQL_UPDATE_ACCOUNT = "UPDATE account SET account_no = ?, branch_id = ?, pin = ?, balance = ? WHERE account_no = ?";

    public List<ShardConfig> determineShards(DbsConnection dbsConnection, DbsStatement dbsStatement, String sql) throws Exception {

        short shardId = -1;
        if (sql.equals(SQL_SELECT_ACCOUNT) || sql.equals(SQL_UPDATE_ACCOUNT )) {
            Number accountNo = (Number) dbsStatement.getParameter(0);
            if (accountNo == null) {
                throw new NullPointerException( "No account number specified"); 
            }

            //System.out.println("account_no = " + accountNo);


//              <vshard id="1" start="0001" end="0999" />
//                <vshard id="2" start="1000" end="1999" />
//                <vshard id="3" start="2000" end="2999" />
//                <vshard id="4" start="3000" end="3999" />
//                <vshard id="5" start="4000" end="4999" />
//                <vshard id="6" start="5000" end="5999" />
//                <vshard id="7" start="6000" end="6999" />
//                <vshard id="8" start="7000" end="7999" />
//                <vshard id="9" start="8000" end="8999" />
//                <vshard id="10" start="9000" end="9999" />

            long l = accountNo.longValue();

            if ((l>=0 && l<=999) || (l>=5000 && l<=5999)) {
                shardId = 1;
            }
            else if ((l>=1000 && l<=1999) || (l>=6000 && l<=6999)) {
                shardId = 2;
            }
            else if ((l>=2000 && l<=2999) || (l>=7000 && l<=7999)) {
                shardId = 3;
            }
            else if ((l>=3000 && l<=3999) || (l>=8000 && l<=8999)) {
                shardId = 4;
            }
            else if ((l>=4000 && l<=4999) || (l>=9000 && l<=9999)) {
                shardId = 5;
            }

            //TODO: should just be able to do this but dbshards isn't fully implemented 
//            shardId = determineShard( "account", accountNo);
        }
        
        if (shardId == -1) {
            System.out.println("Calling DefaultShardResolver for " + sql );
            return super.determineShards(dbsConnection, dbsStatement, sql);
        }

        List<ShardConfig> list = new ArrayList<ShardConfig>(1);
        list.add( dbsConnection.getClientConfig().getShardConfig(shardId) );

        if (logger.isDebugEnabled()) {
            logger.debug( "sql --> " + list );
        }

        return list;

    }

}
