package org.softwarepipelines.framework.connector;

import org.apache.log4j.Logger;

import org.softwarepipelines.config.connector.PipelinesConnectorConfig;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;

import java.util.UUID;
import java.util.concurrent.ArrayBlockingQueue;

/**
 * PipelinesConnector instances serve as messengers between clients and PipelinesDistributors.
 * The PipelinesConnector interface specifies methods to send messages (fire and forget and round-trip).
 * 
 * @author cholm
 */
public abstract class PipelinesConnectorBase implements PipelinesConnector {
    private Logger LOG = Logger.getLogger(PipelinesConnectorBase.class);

    /* Connector Config */
    protected PipelinesConnectorConfig config;

    /* Incoming AbstractMessage Queue */
    protected ArrayBlockingQueue<AbstractMessage> incomingMsgs;
    
    /* Connector Ref */
    protected UUID connectorRef;
    
    public PipelinesConnectorBase() {
    	incomingMsgs = new ArrayBlockingQueue<AbstractMessage>(1);
    	connectorRef = UUID.randomUUID();
    }

    public PipelinesConnectorBase(final PipelinesConnectorConfig aConfig)
        throws PipelineException {
        config = aConfig;
        incomingMsgs = new ArrayBlockingQueue<AbstractMessage>(1);
        connectorRef = UUID.randomUUID();
    }

    public abstract void send(AbstractMessage msg) throws PipelineException;

    public abstract AbstractMessage sendAndReceive(AbstractMessage msg) throws PipelineException;

    protected AbstractMessage waitForResponse() throws PipelineException {
        AbstractMessage response = null;

        while (response == null) {
            try {
                response = this.incomingMsgs.take();
            } catch (InterruptedException ie) {
                LOG.warn("Interrupted while waiting for response AbstractMessage", ie);
            }
        }

        return response;
    }

    public ArrayBlockingQueue<AbstractMessage> getIncomingMsgs() {
        return incomingMsgs;
    }

	public PipelinesConnectorConfig getConfig() {
		return config;
	}

	public void setConfig(PipelinesConnectorConfig config) {
		this.config = config;
	}

	public void setIncomingMsgs(ArrayBlockingQueue<AbstractMessage> incomingMsgs) {
		this.incomingMsgs = incomingMsgs;
	}

	public UUID getConnectorRef() {
		return connectorRef;
	}

	public void setConnectorRef(UUID connectorRef) {
		this.connectorRef = connectorRef;
	}
}
