package org.softwarepipelines.framework.connector;

import java.lang.reflect.Constructor;
import java.util.HashMap;

import org.softwarepipelines.config.connector.PipelinesConnectorConfig;
import org.softwarepipelines.framework.exception.PipelineException;

/**
 * The PipelinesConnectorFactory builds PipelineConnector instances given a PipelinesConnectorConfig instance. 
 * 
 * @author cholm
 *
 */
public class PipelinesConnectorFactory {
	
	/* Connector Type to Class Name Map */
	private static HashMap<String, String> typeToConnClassMap;
	final private static String LOCAL_CONNECTOR_CLASS_NAME = "org.softwarepipelines.framework.connector.LocalPipelinesConnector";
	final private static String SOCKET_CONNECTOR_CLASS_NAME = "org.softwarepipelines.framework.connector.NIOPipelinesConnector";
	final private static String WS_CONNECTOR_CLASS_NAME = "org.softwarepipelines.framework.connector.SpringWSPipelinesConnector";
	static {
		typeToConnClassMap = new HashMap<String, String>();
		typeToConnClassMap.put(PipelinesConnectorConfig.LOCAL_CONNECTOR_TYPE, LOCAL_CONNECTOR_CLASS_NAME);
		typeToConnClassMap.put(PipelinesConnectorConfig.SOCKET_CONNECTOR_TYPE, SOCKET_CONNECTOR_CLASS_NAME);
		typeToConnClassMap.put(PipelinesConnectorConfig.WS_CONNECTOR_TYPE, WS_CONNECTOR_CLASS_NAME);
	}
	
	public static PipelinesConnectorBase getPipelinesConnector(final PipelinesConnectorConfig config) throws PipelineException {
    	
    	// get the connector class name based on the pattern
        String connClassName;
        if (typeToConnClassMap.containsKey(config.getType())) {
        	connClassName = typeToConnClassMap.get(config.getType());
        } else {
        	throw new PipelineException("Connector config type " + config.getType() + " is unknown and thus not mapped to a connector class name");
        }

        try {
            Class<?> theClass = Class.forName(connClassName);
            Constructor<?> ctor = theClass.getConstructor(PipelinesConnectorConfig.class);

            if (ctor == null) {
                throw new PipelineException(theClass.getName() +
                    " does not have a public constructor that accepts a DistributorConfig parameter");
            }

            // build the new instance and set a random UUID
            PipelinesConnectorBase conn = (PipelinesConnectorBase) ctor.newInstance(config);
            return conn;
        } catch (Exception e) {
            throw new PipelineException("Failed to load " + connClassName, e);
        }
    }

}
