package org.softwarepipelines.framework.connector;

import java.rmi.RemoteException;

import org.softwarepipelines.config.connector.PipelinesConnectorConfig;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.springws.PipelinesDistributorRemote;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;

public class SpringWSPipelinesConnector extends PipelinesConnectorBase {
	
	protected PipelinesDistributorRemote distributorProxy;

	public SpringWSPipelinesConnector() {
	}

	public SpringWSPipelinesConnector(PipelinesConnectorConfig config) throws PipelineException {
		// call into the base class
		super(config);
		
		// load the application context
		// TODO: configure?  put as static in helper, or what?
		ApplicationContext ac = new FileSystemXmlApplicationContext("config/connectorContext.xml");
		
		// and then get the proxy 
		distributorProxy = (PipelinesDistributorRemote) ac.getBean( "distributorProxy" );
	}

	@Override
	public AbstractMessage sendAndReceive(AbstractMessage msg) throws PipelineException {
		// set the connector reference
		msg.setConnectorRef(this.getConnectorRef().toString());
		
		// and invoke the service through the proxy
		try {
			return distributorProxy.invokeReturn(msg);
		} catch (RemoteException re) {
			throw new PipelineException(re.getMessage(), re);
		}
	}

	@Override
	public void send(AbstractMessage msg) throws PipelineException {
		// set the connector reference
		msg.setConnectorRef(this.getConnectorRef().toString());
		
		// invoke the service through the proxy
		try {
			distributorProxy.invokeVoid(msg);
		} catch (RemoteException re) {
			throw new PipelineException(re.getMessage(), re);
		}

	}
	
	public void shutdown() throws PipelineException {
		// nothing to do here for now?
	}

}
