package org.softwarepipelines.framework.distributor;

import org.softwarepipelines.config.distributor.PipelinesDistributorConfig;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;


/**
 * Interface for a generic PipelinesDistributor.  Includes methods for configuring the distributor and
 * distributing a message to the appropriate PipelinesRouter.
 *
 * @author cholm
 *
 */
public interface PipelinesDistributor {
    /**
     * Configure the distributor based on the given PipelinesDistributorConfig.
     */
    public void configure(final PipelinesDistributorConfig config) throws PipelineException;

    /**
     * Distribute the incoming Message to the appropriate PipelinesRouter.
     * @param msg the Message to distribute
     */
    public void distribute(final AbstractMessage msg) throws PipelineException;
}
