package org.softwarepipelines.framework.distributor;

import java.io.File;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.framework.util.PipelinesHelper;

/**
 * Simple utility to start a <code>PipelinesDistributor</code> instance locally from the command line.
 * 
 * @author cholm
 *
 */
public class PipelinesDistributorRunner {
	private final static Logger LOG = Logger.getLogger(PipelinesDistributorRunner.class);
	
	// the distributor to start
	private static PipelinesDistributorBase distributor;

	/**
	 * Start a <code>PipelinesDistributor</code> instance, configured in the configuration file whose path is provided.
	 * 
	 * @param args in order, the configuration file path and then the name of the distributor to start 
	 */
	public static void main(String[] args) throws Exception {
		String configFilePath, distributorName;
		PipelinesMasterConfig masterConfig;
		
		if (args.length != 2) {
			LOG.error("Usage: PipelinesDistributorRunner [configuration file path] [distributor name]");
		}
		
		// load the configuration
		configFilePath = args[0];
		LOG.debug("Loading PipelinesMasterConfig from file: " + configFilePath);
		masterConfig = new PipelinesMasterConfig(new File(configFilePath));
		
		// get and start the distributor
		distributorName = args[1];
		LOG.debug("Starting distributor named: " + distributorName);
		distributor = PipelinesHelper.getDistributorByName(distributorName, masterConfig);
		distributor.start();
	}
	
	/**
	 * Shut down the singleton distributor instance.
	 */
	protected void finalize() throws Exception {
		distributor.shutdown();
	}

}
