package org.softwarepipelines.framework.distributor;

import org.apache.log4j.Logger;

import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.router.PipelinesRouter;

/**
 * 
 * @author cholm
 */
public class PushPatternDistributor extends PipelinesDistributorBase {
    private static final Logger LOG = Logger.getLogger(PushPatternDistributor.class);

    public PushPatternDistributor() throws PipelineException {
        super();
    }

    /**
     * Distribute the message to the appropriate Router; make synchronized to allow for multiple local connectors.
     */
    public synchronized void distribute(final AbstractMessage msg) throws PipelineException {
    	PipelinesRouter router;
        
        // log at the debug level the message we are distributing
        LOG.debug("Distributing message: " + msg);
        
        // Lookup the appropriate router instance
        if (!this.getServiceToRouterMap().containsKey(msg.getServiceName())) {
        	throw new PipelineException("No PipelinesRouter instance for service named " + msg.getServiceName());
        }
        router = this.getServiceToRouterMap().get(msg.getServiceName()); 
        
        // and then route the message
        router.route(msg);
    }    
}
