package org.softwarepipelines.framework.message;

import java.lang.reflect.Constructor;

import org.softwarepipelines.config.message.MessageConfig;
import org.softwarepipelines.framework.exception.PipelineException;

public class MessageFactory {
	
	public static AbstractMessage getMessage(final MessageConfig config, final String name) throws PipelineException {
		try {
            Class<?> theClass = config.getType();
            Constructor<?> ctor = theClass.getConstructor(String.class);

            if (ctor == null) {
                throw new PipelineException("Message " +
                    theClass.getName() +
                    " does not have a public constructor that accepts " +
                    "(String) parameters");
            }

            return (AbstractMessage) ctor.newInstance(name);
        } catch (Exception e) {
            throw new PipelineException("Failed to load AbstractMessage " +
                config.getType().getName(), e);
        }
	}
	
	public static AbstractMessage getMessage(final String className, final String serviceName) throws PipelineException {
		try {
            Class<?> theClass = Class.forName(className);
            Constructor<?> ctor = theClass.getConstructor(String.class);

            if (ctor == null) {
                throw new PipelineException("Message " +
                    theClass.getName() +
                    " does not have a public constructor that accepts " +
                    "(String) parameters");
            }

            return (AbstractMessage) ctor.newInstance(serviceName);
        } catch (Exception e) {
            throw new PipelineException("Failed to load AbstractMessage " +
                className, e);
        }
	}

}
