package org.softwarepipelines.framework.nio;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.softwarepipelines.config.distributor.PipelinesDistributorConfig;
import org.softwarepipelines.framework.exception.PipelineException;

public class MessageBrokerFactory {
	
	private static HashMap<String, ServerMessageBroker> serverBrokerMap;
	private static List<String> serverBrokerNames;
	private static HashMap<String, ClientMessageBroker> clientBrokerMap;
	private static List<String> clientBrokerNames;
	static {
		serverBrokerMap = new HashMap<String, ServerMessageBroker>();
		clientBrokerMap = new HashMap<String, ClientMessageBroker>();
		serverBrokerNames = new ArrayList<String>();
		clientBrokerNames = new ArrayList<String>();
	}
	
	public static ServerMessageBroker getMessageBroker(final PipelinesDistributorConfig distConfig) throws PipelineException {
		
		// check the distributor name, see if we already have a server broker instantiated it; if so, return it
		if (serverBrokerMap.containsKey(distConfig.getName())) {
			return serverBrokerMap.get(distConfig.getName());
		}
		
		// otherwise, isntantiate the broker and add a map entry
		ServerMessageBroker broker = new ServerMessageBroker();
		broker.configure(distConfig);
		serverBrokerMap.put(distConfig.getName(), broker);
		serverBrokerNames.add(distConfig.getName());
		return broker;
	}
	
	public static ClientMessageBroker getClientMessageBroker(final PipelinesDistributorConfig distConfig) throws PipelineException {
		// check the distributor name, see if we already have a server broker instantiated it; if so, return it
		if (clientBrokerMap.containsKey(distConfig.getName())) {
			return clientBrokerMap.get(distConfig.getName());
		}
		
		// otherwise, isntantiate the broker and add a map entry
		ClientMessageBroker broker = new ClientMessageBroker();
		broker.configure(distConfig);
		clientBrokerMap.put(distConfig.getName(), broker);
		clientBrokerNames.add(distConfig.getName());
		return broker;
	}
	
	public static void shutdownServerBroker(final String brokerName) {
		for (String name : serverBrokerNames) {
			if (serverBrokerMap.containsKey(name)) {
				serverBrokerMap.remove(name);
				return;
			} 
		}
	}
		
	public static void shutdownClientBroker(final String brokerName) {
		for (String name : clientBrokerNames) {
			if (clientBrokerMap.containsKey(name)) {
				clientBrokerMap.remove(name);
				return;
			} 
		}
	}
	
	public static void shutdownBroker(final MessageBrokerBase broker) {
		//shut down the broker
		if (broker instanceof ServerMessageBroker) {
			shutdownServerBroker(broker.getDistConfig().getName());
		} else {
			shutdownClientBroker(broker.getDistConfig().getName());
		} 
	}
	
	public synchronized static void shutdownAll() throws PipelineException {
		MessageBrokerBase broker;
		
		// shut each broker down
		for (String name : serverBrokerNames) {
			if (serverBrokerMap.containsKey(name)) {
				broker = serverBrokerMap.remove(name);
				broker.shutdown();
			} 
		}
		for (String name : clientBrokerNames) {
			if (clientBrokerMap.containsKey(name)) {
				broker = clientBrokerMap.remove(name);
				broker.shutdown();
			} 
		}
	}
}
