package org.softwarepipelines.framework.pipeline;

import org.softwarepipelines.config.pipeline.PipelineConfig;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;

/**
 * Interface for a generic Pipeline.
 * 
 * Includes methods for configuring the pipeline and invoking the service with which the pipeline is associated
 * given some input AbstractMessage.
 * 
 * @author cholm
 */
public interface Pipeline {
	
	/**
     * Configure the pipeline based on the given PipelineConfig.
     */
    public void configure(final PipelineConfig config) throws PipelineException;

    /**
     * Invoke the service with which this pipeline is associated with the given AbstractMessage as input.
     * 
     * @param msg the AbstractMessage with which we will invoke the service
     */
    public void invokeService(final AbstractMessage msg) throws PipelineException;

}
