package org.softwarepipelines.framework.pipeline;

import java.lang.reflect.Constructor;

import org.softwarepipelines.config.pipeline.PipelineConfig;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.util.Counter;

public class PipelineFactory {

	public static PipelineImpl getPipeline(final PipelineConfig config, 
				final PipelinesDistributorBase dist,
				Counter counter) throws PipelineException {
		
		String pipelineClassName = "org.softwarepipelines.framework.pipeline.PipelineImpl";
    	
        try {
            Class<?> theClass = Class.forName(pipelineClassName);
            Constructor<?> ctor = theClass.getConstructor(PipelineConfig.class, PipelinesDistributorBase.class, Counter.class);

            if (ctor == null) {
                throw new PipelineException("Pipeline " +
                    theClass.getName() +
                    " does not have a public constructor that accepts " +
                    "(PipelineConfig, PipelinesDistributorBase, Counter) parameters");
            }

            return (PipelineImpl) ctor.newInstance(config, dist, counter);
        } catch (Exception e) {
            throw new PipelineException("Failed to load Pipeline " +
                pipelineClassName, e);
        }
    }
}
