package org.softwarepipelines.framework.router;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.router.PipelinesRouterConfig;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.pipeline.Pipeline;
import org.softwarepipelines.framework.pipeline.PipelineImpl;

public class ContentBasedRouterImpl extends PipelinesRouterBase {
	private final static Logger LOG = Logger.getLogger( ContentBasedRouterImpl.class );
	
	public ContentBasedRouterImpl(){
	}
	
	public ContentBasedRouterImpl(final PipelinesRouterConfig config, final PipelinesDistributorBase dist) throws PipelineException {
		super(config, dist);
	}

	public void route(final AbstractMessage msg) throws PipelineException {
		// check to see if we have a String match or an int match range
		// XXX for now, just look at the first pipeline; this sucks as a way to do it, we should configure a type
		// on the message along with the pipeline key or something along those lines
		if (config.getServiceConfig().getPipelines().get(0).getMatch() != null) {
			routeStringKey(msg);
		} else {
			routeLongKey(msg);
		}
	}
	
	private void routeStringKey(final AbstractMessage msg) throws PipelineException {
		// get the pipeline key value
		String keyVal = (String) getPipelineKey(msg);
		
		// and match it to a pipeline
		PipelineImpl plImpl;
		for (Pipeline pipeline : this.getPipelines()) {
			plImpl = (PipelineImpl) pipeline;
			if (plImpl.getConfig().getMatch().matcher(keyVal).matches()) {
				LOG.debug("Invoking service for message with key value " + keyVal + " in pipeline " + plImpl.getConfig().getName() + 
						" with match pattern " + plImpl.getConfig().getMatch().pattern());
				plImpl.invokeService(msg);
				return;
			}
		}
		
		// if we get to this point, then we didn't find a matching pipeline, throw
		throw new PipelineException("No pipeline regexp matched key value " + keyVal + 
				"!\nPipelines: " + config.getServiceConfig().getPipelines());
	}
	
	private void routeLongKey(final AbstractMessage msg) throws PipelineException {
		// get the pipeline key value
		Object keyValObj = getPipelineKey(msg);
		
		long keyVal;
		if (keyValObj instanceof Integer) {
			keyVal = ((Integer)keyValObj).longValue();
		} else if (keyValObj instanceof Double) {
			keyVal = ((Double)keyValObj).longValue();
		} else if (keyValObj instanceof Long) {
			keyVal = ((Long)keyValObj).longValue();
		} else {
			throw new PipelineException("Unsupported int-expr range endpoint type " + keyValObj.getClass().getName() + 
					"!\nSupported types are [" + Long.class.getName() + ", " + Integer.class.getName() + ", " + Double.class.getName() + "]");
		}
		
		// and match it to a pipeline
		PipelineImpl plImpl;
		for (Pipeline pipeline : this.getPipelines()) {
			plImpl = (PipelineImpl) pipeline;
			if (plImpl.getConfig().getMatchFrom() <= keyVal && plImpl.getConfig().getMatchTo() >= keyVal) {
				LOG.debug("Invoking service for message with key value " + keyVal + " in pipeline " + plImpl.getConfig().getName() + 
						" with match range " + plImpl.getConfig().getMatchFrom() + " - " + plImpl.getConfig().getMatchTo());
				plImpl.invokeService(msg);
				return;
			}
		}
		
		// if we get to this point, then we didn't find a matching pipeline, throw
		throw new PipelineException("No pipeline range matched key value " + keyVal + 
				"!\nPipelines: " + config.getServiceConfig().getPipelines());
	}
}
