package org.softwarepipelines.framework.router;

import org.softwarepipelines.config.router.PipelinesRouterConfig;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;

/**
 * Interface for a <code>PipelinesRouter</code>.  Includes methods for configuring the router and routing a message to the appropriate pipeline.
 * <br><br>
 * Custom routers should extend <code>org.softwarepipelines.framework.router.PipelinesRouterBase</code> class, which implements this
 * interface, and override the <code>route(AbstractMessage msg)</code> method to implement custom routing logic.
 *
 * @author cholm
 *
 */
public interface PipelinesRouter {
    /**
     * Configure the Router instance according to the given PipelinesRouterConfig.
     */
    public void configure(final PipelinesRouterConfig config) throws PipelineException;

    /**
     * Route the incoming message to the appropriate Pipeline.
     * @param msg the AbstractMessage to distribute
     */
    public void route(final AbstractMessage msg) throws PipelineException;
}
