package org.softwarepipelines.framework.router;

import java.lang.reflect.Constructor;

import org.softwarepipelines.config.router.PipelinesRouterConfig;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;

public class PipelinesRouterFactory {
	
	public static PipelinesRouterBase getPipelinesRouter(final PipelinesRouterConfig config, final PipelinesDistributorBase dist) throws PipelineException {
    	
        String rtrClassName = config.getClassName();

        if (rtrClassName == null) {
            throw new PipelineException(
                "No pipeline service class name specified on the config: " +
                config);
        }

        try {
            Class<?> theClass = Class.forName(rtrClassName);
            Constructor<?> ctor = theClass.getConstructor(PipelinesRouterConfig.class, PipelinesDistributorBase.class);

            if (ctor == null) {
                throw new PipelineException("PipelinesRouter " +
                    theClass.getName() +
                    " does not have a public constructor that accepts " +
                    "(PipelinesRouterConfig) parameter");
            }

            return (PipelinesRouterBase) ctor.newInstance(config, dist);
        } catch (Exception e) {
            throw new PipelineException("Failed to load PipelineRouter " +
                rtrClassName, e);
        }
	}
}
