package org.softwarepipelines.framework.router;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.router.PipelinesRouterConfig;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.pipeline.PipelineImpl;

public class RoundRobinRouterImpl extends PipelinesRouterBase {
	private static final Logger LOG = Logger.getLogger( RoundRobinRouterImpl.class );

	/* Round Robin Pipeline Counter */
	private int numPipelines;
	private int currPipeline;
	
	public RoundRobinRouterImpl() {
		super();
	}
	
	public RoundRobinRouterImpl(final PipelinesRouterConfig aConfig, final PipelinesDistributorBase dist) throws PipelineException {
		super(aConfig, dist);
	}
	
	public void configure(final PipelinesRouterConfig aConfig) throws PipelineException {
		super.configure(aConfig);
		
		currPipeline = 0;
		numPipelines = this.pipelines.size();
	}

	public void route(final AbstractMessage msg) throws PipelineException {
		PipelineImpl pipeline;
		
		// retrieve the next pipeline
		pipeline = (PipelineImpl) pipelines.get(currPipeline % numPipelines);
		
		// invoke the service
		LOG.debug("Round robin invoking service for message in pipeline " + pipeline.getConfig().getName());
		pipeline.invokeService(msg);
		
		// and increment the counter
		currPipeline ++;
	}

}
