package org.softwarepipelines.framework.service;

import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;

/**
 * Interface for a generic <code>PipelineService</code>.  Includes a method to process an incoming <code>AbstractMessage</code>.<br>
 * <br>
 * Custom <code>PipelineServices</code> must extend the abstract <code>PipelineServiceBase</code> class, which implements this interface.
 * 
 * @author cholm
 */
public interface PipelineService {
	
    /**
     * Process an incoming message.
     * 
     * If you wish to return a message from this service invoke to the invoking connector, call the<br>
     * <br> 
     * <code>PipelineServiceBase.sendResultMessage(AbstractMessage result)</code>
     * <br>
     * <br>method in the base service class (which custom services must extend).
     * 
     * @param msg the message to process
     * @throws PipelineException in case of error or failure
     */
	public void processMessage(AbstractMessage msg) throws PipelineException;
}
