package org.softwarepipelines.framework.service;

import org.softwarepipelines.config.pipeline.PipelineConfig;
import org.softwarepipelines.config.service.PipelineServiceConfig;
import org.softwarepipelines.framework.exception.PipelineException;

import java.lang.reflect.Constructor;


public class PipelineServiceFactory {
	
	public static PipelineServiceBase getPipelineService(final PipelineServiceConfig serviceConfig) throws PipelineException {
    	
        String serviceClassName = serviceConfig.getClassName();

        if (serviceClassName == null) {
            throw new PipelineException(
                "No pipeline service class name specified on the config: " +
                serviceConfig);
        }

        try {
            Class<?> theClass = Class.forName(serviceClassName);
            Constructor<?> ctor = theClass.getConstructor(PipelineServiceConfig.class);

            if (ctor == null) {
                throw new PipelineException("PipelineService " +
                    theClass.getName() +
                    " does not have a public constructor that accepts " +
                    "(PipelineServiceConfig) parameters");
            }

            return (PipelineServiceBase) ctor.newInstance(serviceConfig);
        } catch (Exception e) {
            throw new PipelineException("Failed to load PipelineService " +
                serviceClassName, e);
        }
    }
    
    public static RelayService getSocketRelayService(final PipelineConfig plConfig) throws PipelineException {
    	
    	return new RelayService(plConfig);
    }
}
