package org.softwarepipelines.framework.connector;

import junit.framework.TestCase;

import org.apache.log4j.Logger;

import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.util.PipelinesHelper;
import org.softwarepipelines.framework.util.PipelinesTestHelper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.io.FileNotFoundException;
import java.io.IOException;

public class NIOPipelinesConnectorTestCase extends TestCase {
    private static final Logger LOG = Logger.getLogger(LocalPipelinesConnectorTestCase.class);
    
    // Application Context
    private ApplicationContext ac;
    
    // framework components
    private PipelinesMasterConfig config;
    private PipelinesConnector connector;
    private PipelinesDistributorBase distributor;
    
    public NIOPipelinesConnectorTestCase(String name) {
        super(name);
        
     	// load the framework test application context
        ac = new ClassPathXmlApplicationContext(PipelinesTestHelper.FRAMEWORK_TEST_CONTEXT_FILE);
    }

    public void setUp() throws PipelineException, FileNotFoundException, IOException, ConfigException {
    	// TODO: inject distributor name with Spring app context
    	
        // setup up the service invoker
    	config = (PipelinesMasterConfig) ac.getBean("singleTierRRConfig");
    	distributor = PipelinesHelper.getDistributorByName("localDistributor", config);
    	distributor.start();
        
        // and check out a socket connector
        connector = PipelinesHelper.getConnectorByName(distributor.getConfig().getName(), "socketConnector", config);
    }

    public void tearDown() throws InterruptedException, PipelineException {
        // shut down the distributor and the connector
    	connector.shutdown();
    	distributor.shutdown();
        try {Thread.sleep(1000);} catch (Exception e){}
    }

    public void testSend() throws PipelineException {
        // test that we can successfully invoke a void pipeline service
    	HelloMessage msg = new HelloMessage();
    	msg.setServiceName("helloService");
    	msg.setName("Andy");
        connector.send(msg);
        
        // wait for the hello service invoke to be enqueued
        try {Thread.sleep(100);} catch (Exception e) {}
    }

    public void testSendAndReceive() throws PipelineException {
    	AbstractMessage resp;
    	
        // build a message with a void destination service
    	HelloMessage msg = new HelloMessage();
    	msg.setServiceName("helloService");
    	msg.setName("Steven");
    	
    	// and test that we time out waiting for the response
    	try {
    		resp = connector.sendAndReceive(msg);
    	} catch (PipelineException pe) {
    		assertEquals(PipelineException.RESP_TIMEOUT_MSG + "\tSent message: " + msg, 
    				pe.getMessage());
    	} catch (Exception e) {
    		fail("Should have caught PipelineException! Caught: " + e);
    	}
    	
    	// set the return dest service
    	msg.setServiceName("helloRetService");

    	// and finally, test that we can invoke the return service and get a valid response
        resp = connector.sendAndReceive(msg);
        assertNotNull(resp);
        assertEquals(msg.getServiceName(), resp.getServiceName());
        LOG.info("Got response: " + resp);
    }
}
