package org.softwarepipelines.framework.connector;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.util.PipelinesHelper;
import org.softwarepipelines.framework.util.PipelinesTestHelper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import junit.framework.TestCase;

public class SpringWSConnectorTestCase extends TestCase {
	private static final Logger LOG = Logger.getLogger(SpringWSConnectorTestCase.class);
	
	// Application Context
    private ApplicationContext ac;
	 
    private PipelinesMasterConfig config;
    private PipelinesConnector connector;

	public SpringWSConnectorTestCase(String name) {
		super(name);
		
		// load the framework test application context
        ac = new ClassPathXmlApplicationContext(PipelinesTestHelper.FRAMEWORK_TEST_CONTEXT_FILE);
	}
	
	public void setUp() throws PipelineException, FileNotFoundException, IOException, ConfigException {
	    // setup up the service invoker
		config = (PipelinesMasterConfig) ac.getBean("springWsConfig");
		connector = PipelinesHelper.getConnectorByName("springWsDistributor", "wsConnector", config); 
	}
	
	public void testVoidServiceInvoke() throws PipelineException {
		// build a message with a void destination service
    	HelloMessage msg = new HelloMessage();
    	msg.setServiceName("helloService");
    	msg.setName("Colin");
    	
    	// and invoke it
		connector.send(msg);
		LOG.debug("Invoked service helloService...");
	}
	
	public void testReturnServiceInvoke() throws PipelineException {
		AbstractMessage resp;
		
		// build a message with a void destination service
    	HelloMessage msg = new HelloMessage();
    	msg.setServiceName("helloService");
    	msg.setName("Colin");
		
    	// and try to invoke it
    	try {
    		resp = connector.sendAndReceive(msg);
    	} catch (PipelineException pe) {
    		assertTrue("Message should contain:\n" + PipelineException.NOT_A_RETURN_SERVICE_MSG + 
    				"\n\tDistributor: springWsDistributor" + 
    				"\n\tService: " + msg.getServiceName() + "\nActual message: " + pe.getMessage(),
    				pe.getMessage().contains(PipelineException.NOT_A_RETURN_SERVICE_MSG + 
    				"\n\tDistributor: springWsDistributor" + 
    				"\n\tService: " + msg.getServiceName()));
    	} catch (Exception e) {
    		fail("Expecting to catch PipelineException, caught: " + e);
    	}
    	
    	// set the correct service name
    	msg.setServiceName("helloRetService");
    	
    	// and then test that we get a valid response
    	resp = connector.sendAndReceive(msg);
		assertNotNull(resp);
		assertTrue(resp instanceof HelloMessage);
		HelloMessage helloResp = (HelloMessage) resp;
		assertEquals("helloRetService", helloResp.getServiceName());
		assertFalse(helloResp.getName(),equals(msg.getName()));
		LOG.debug("Invoked helloRetService, got response: " + resp);
	}

}
