package org.softwarepipelines.framework.distributor;

import java.io.FileNotFoundException;
import java.io.IOException;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.softwarepipelines.framework.util.PipelinesHelper;
import org.softwarepipelines.framework.util.PipelinesTestHelper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * 
 * @author cholm
 *
 */
public class MultiTierDistributorTestCase extends TestCase {
	private static final Logger LOG = Logger.getLogger(MultiTierDistributorTestCase.class);	
	
	/* Application Context */
	private ApplicationContext ac;
	
	/* Pipelines Framework Components */ 
    private PipelinesMasterConfig masterConfig;
    private PipelinesDistributorBase dist1, dist2;
    private PipelinesConnector testConnector;

	public MultiTierDistributorTestCase(String name) {
		super(name);
		
		ac = new ClassPathXmlApplicationContext(PipelinesTestHelper.FRAMEWORK_TEST_CONTEXT_FILE);
	}
	
	public void tearDown() throws PipelineException {
		testConnector.shutdown();
		dist1.shutdown();
		dist2.shutdown();
	}
	
	private void setupDistributors(final String beanName, final String connName) throws PipelineException, ConfigException, FileNotFoundException, IOException {
    	// setup up the service invoker
    	masterConfig = (PipelinesMasterConfig) ac.getBean(beanName);
    	
    	// get and start both distributors
    	dist1 = PipelinesHelper.getDistributorByName("relayDistributor", masterConfig);
    	dist2 = PipelinesHelper.getDistributorByName("invokeDistributor", masterConfig);
    	
    	// start in reverse order
    	dist1.start();
    	dist2.start();
    	
    	// and then get the connector
    	testConnector = PipelinesHelper.getConnectorByName(dist1.getConfig().getName(), connName, masterConfig);
    }
	
	/**
	 * 
	 */
	public void testLocalToSocketTwoTierDistributor() throws PipelineException, ConfigException, FileNotFoundException, IOException {
		//AbstractMessage resp;
		
		// setup the first tier broker and the second tier distributor
		setupDistributors("twoTierConfig", "localConnector");
		
		// and use the test connector to invoke a void service
		HelloMessage msg = new HelloMessage();
    	msg.setServiceName("helloService");
    	msg.setName("Nick");
		testConnector.send(msg);
		LOG.debug("Invoked service " + msg.getServiceName() + " with message: " + msg);
		
		// and then a return service
		//XXX: No local connectors allowed at the client side of a return service!
		/*msg = new HelloMessage();
    	msg.setServiceName("helloRetService");
    	msg.setName("Randy");
		resp = testConnector.sendAndReceive(msg);
		assertNotNull(resp);
		assertTrue(resp instanceof HelloMessage);
		HelloMessage helloResp = (HelloMessage) resp;
		assertEquals("Software Pipelines", helloResp.getName());
		LOG.debug("Invoked return relay service, got response: " + resp);*/
	}
	
	/**
	 * 
	 */
	public void testSocketToSocketTwoTierDistributor() throws PipelineException, ConfigException, FileNotFoundException, IOException {
		AbstractMessage resp;
		
		// setup the first tier broker and the second tier distributor
		setupDistributors("twoTierConfig", "socketConnector");
		
		// and use the test connector to invoke a void service
		org.softwarepipelines.examples.hello.HelloMessage msg = new org.softwarepipelines.examples.hello.HelloMessage();
    	msg.setServiceName("helloService");
    	msg.setName("Nick");
		testConnector.send(msg);
		LOG.debug("Invoked service " + msg.getServiceName() + " with message: " + msg);
		
		// and then a return service
		msg = new org.softwarepipelines.examples.hello.HelloMessage();
    	msg.setServiceName("helloRetService");
    	msg.setName("Randy");
		resp = testConnector.sendAndReceive(msg);
		assertNotNull(resp);
		assertTrue(resp instanceof HelloMessage);
		HelloMessage helloResp = (HelloMessage) resp;
		assertEquals("Software Pipelines", helloResp.getName());
		LOG.info("Invoked return relay service, got response: " + resp);
	}

}
