package org.softwarepipelines.framework.perf;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.jperf.PerfTest;
import org.jperf.PerfTestRunner;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.connector.PipelinesConnector;
import org.softwarepipelines.framework.distributor.PipelinesDistributorBase;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.util.PipelinesHelper;
import org.softwarepipelines.framework.util.PipelinesTestHelper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public class PipelinesConnectorPerfTest implements PerfTest {
private final static Logger LOG = Logger.getLogger(PipelinesConnectorPerfTest.class);
	
	// Pipelines Framework Components
	protected static PipelinesMasterConfig masterConfig;
	protected static PipelinesDistributorBase distributor;
    protected PipelinesConnector connector;
    protected static String connectorName;
    protected static boolean isResponse;
	
	public void setUp() throws PipelineException {
		connector = PipelinesHelper.getConnectorByName(distributor.getConfig().getName(), connectorName, masterConfig);
    }

    public void tearDown() throws PipelineException {
    	connector.shutdown();
    }

    public void test() throws Exception {
    	HelloMessage msg, resp;
    	
    	// build the message
    	msg = new HelloMessage();
    	msg.setName("Colin");
    	LOG.debug("Invoking service " + msg.getServiceName() + " with message " + msg);
    	
    	if (isResponse) {
    		msg.setServiceName("helloRetService");
    		resp = (HelloMessage) connector.sendAndReceive(msg);
    		LOG.debug("Received response: " + resp);
    	} else {
    		msg.setServiceName("helloService");
    		connector.send(msg);
    	}
    }

    /**
     * @param args
     */
    public static void main(String[] args) throws IOException, PipelineException, ConfigException {
    	String configBean, distributorName;
		
		/* Get the parameters.*/
		if(args.length != 4) {
			throw new PipelineException("Usage: HelloPipelinesClient [pipelinesConfig bean name] [distributorName] [connectorName] [isResponseTest]");
		}
		configBean = args[0];
		distributorName = args[1];
		connectorName = args[2];
		isResponse = Boolean.parseBoolean(args[3]);
				
		/* Load the Pipelines Configuration.*/
		ApplicationContext ac = new ClassPathXmlApplicationContext(PipelinesTestHelper.FRAMEWORK_TEST_CONTEXT_FILE);
		masterConfig = (PipelinesMasterConfig) ac.getBean(configBean);
		
		/* Setup the destination distributor */
		distributor = PipelinesHelper.getDistributorByName(distributorName, masterConfig);
		distributor.start();
		
		/* Set up the PerfTest runner for 6 threads, 4 seconds between threads.*/
		PerfTestRunner r = new PerfTestRunner();
        r.setMaxClient(6);
        r.setTestPeriod(4000);
        
        /* Specify the results output file.*/
        r.setResultFile(new File("results/test/PipelinesConnectorPerfTest-results.xml"));
        
        /* Run the test.*/
        r.run(PipelinesConnectorPerfTest.class);
        
        /* Shutdown the brokers and executor threads.*/
        distributor.shutdown();
    }
}
