package org.softwarepipelines.framework.util;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.softwarepipelines.config.PipelinesMasterConfig;
import org.softwarepipelines.config.distributor.PipelinesDistributorConfig;
import org.softwarepipelines.config.exception.ConfigException;
import org.softwarepipelines.examples.hello.HelloMessage;
import org.softwarepipelines.framework.exception.PipelineException;
import org.softwarepipelines.framework.message.AbstractMessage;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import junit.framework.TestCase;

public class PipelinesHelperTestCase extends TestCase {
	private final static Logger LOG = Logger.getLogger(PipelinesHelperTestCase.class);
	
	private ApplicationContext ac = new ClassPathXmlApplicationContext(PipelinesTestHelper.FRAMEWORK_TEST_CONTEXT_FILE);

	private static int THRU_ITS = 500000;
	private static String DUMMY_SERVICE = "dummyService";
	private static HashMap<String, Class<?>> dummyMsgClassMap;
	static {
		dummyMsgClassMap = new HashMap<String, Class<?>>();
		dummyMsgClassMap.put(DUMMY_SERVICE, AbstractMessage.class);
	}

	public PipelinesHelperTestCase(String name) {
		super(name);
	}
	
	public void testReadAndWriteMessage() throws PipelineException, ConfigException, IOException {
		// load a Distributor instance
		PipelinesMasterConfig masterConf = (PipelinesMasterConfig) ac.getBean("singleTierCBConfig");
		PipelinesDistributorConfig distConfig = masterConf.getDistributors().get(0);
		
		// build a HelloMessage
		HelloMessage msg = new HelloMessage();
		msg.setConnectorRef(UUID.randomUUID().toString());
		msg.setServiceName("helloService");
		msg.setName("Colin");
		
		// and write it into the byte buffer
		ByteBuffer msgBuffer = ByteBuffer.allocate(2048);
		MessageHelper.writeMessageToBuffer(msg, msgBuffer);
		
		// now, read it in from the buffer AS AN ABSTRACT MESSAGE
		msgBuffer.flip();
		AbstractMessage msg2 = MessageHelper.readMessageFromBuffer(msgBuffer, distConfig.getServiceToMsgClassMap());
		assertEquals(msg, msg2);
	}
	
	/**
	 * 
	 * @throws PipelineException
	 * @throws ConfigException
	 * @throws IOException
	 */
	public void testWriteThru() throws PipelineException, ConfigException, IOException {
		// build a HelloMessage
		HelloMessage msg = new HelloMessage();
		msg.setConnectorRef(UUID.randomUUID().toString());
		msg.setServiceName("helloService");
		msg.setName("Colin");
		
		// and write it into the byte buffer
		ByteBuffer msgBuffer = ByteBuffer.allocate(2048);
		long start = System.currentTimeMillis();
		for (int i = 0; i < THRU_ITS; i++) {
			MessageHelper.writeMessageToBuffer(msg, msgBuffer);
		}
		long end = System.currentTimeMillis();
		LOG.info(THRU_ITS + " calls to writeMessageToBuffer took " + (end-start) + " ms\t(" + 
				((float)1000*(float)THRU_ITS)/((float)(end-start)) + " TPS)");
	}
	
	/**
	 * 
	 * @throws PipelineException
	 * @throws ConfigException
	 * @throws IOException
	 */
	public void testReadThru() throws PipelineException, ConfigException, IOException {
		// load a Distributor instance
		PipelinesMasterConfig masterConf = (PipelinesMasterConfig) ac.getBean("singleTierCBConfig");
		PipelinesDistributorConfig distConfig = masterConf.getDistributors().get(0);
		
		// build a HelloMessage
		HelloMessage msg = new HelloMessage();
		msg.setConnectorRef(UUID.randomUUID().toString());
		msg.setServiceName("helloService");
		msg.setName("Colin");
		
		// and write it into the byte buffer
		ByteBuffer msgBuffer = ByteBuffer.allocate(2048);
		MessageHelper.writeMessageToBuffer(msg, msgBuffer);
		msgBuffer.flip();
		
		AbstractMessage msg2 = null;
		long start = System.currentTimeMillis();
		for (int i = 0; i < THRU_ITS; i++) {
			msg2 = MessageHelper.readMessageFromBuffer(msgBuffer, distConfig.getServiceToMsgClassMap());
			msgBuffer.rewind();
		}
		long end = System.currentTimeMillis();
		assertNotNull(msg2);
		LOG.info(THRU_ITS + " calls to readMessageFromBuffer took " + (end-start) + " ms\t(" + 
				((float)1000*(float)THRU_ITS)/((float)(end-start)) + " TPS)");
	}
	
	/**
	 * Tests writing a message into a buffer that is too small.
	 * 
	 * Ensures that the buffer is reallocated with an additional 1K of memory.
	 * @throws PipelineException
	 *
	public void testWriteReallocates() throws PipelineException {
		int smallCap = 10;
		int expBigCap = 1034;
		
		// create a message with a service name that is larger than the buffer's capacity
		HelloMessage msg = new HelloMessage();
		msg.setServiceName("helloService");
		msg.setName("Colin");
		ByteBuffer smallBuf = ByteBuffer.allocate(smallCap);
		
		// then call to write the message into the buffer
		smallBuf = MessageHelper.writeMessageToBuffer(msg, smallBuf);
		assertEquals(expBigCap, smallBuf.capacity());
		
		// make sure we can still read the message
		AbstractMessage msg2 = MessageHelper.readMessageFromBuffer(smallBuf, dummyMsgClassMap);
		assertEquals(msg, msg2);
		
		// now create a message with a short enough service name, but contents too long
		expBigCap = 1024 + 44;
		smallBuf = ByteBuffer.allocate(44);
		smallBuf = MessageHelper.writeMessageToBuffer(msg, smallBuf);
		assertEquals(expBigCap, smallBuf.capacity());
		
		// make sure we can still read the message
		msg2 = MessageHelper.readMessageFromBuffer(smallBuf, dummyMsgClassMap);
		assertEquals(msg, msg2);
	}*/
}
