package org.softwarepipelines.framework.uuid;

import java.util.UUID;

import org.apache.log4j.Logger;

public class UUIDPerformance {
	private static Logger LOG = Logger.getLogger( UUIDPerformance.class );
	
	final private static int NUM_ITS = 1000000;
	
	public static void main (String [] args) {
		UUID uuid = null;
		long start, end, total;
		String aSrvName = "serviceName";
		
		// test creating from byte [] gen'd from string name
		start = System.currentTimeMillis();
		for (int i = 0; i < NUM_ITS; i++) {
			uuid = UUID.nameUUIDFromBytes(aSrvName.getBytes());
		}
		end = System.currentTimeMillis();
		total = end - start;
		LOG.info("Creating " + NUM_ITS + " UUIDs via the nameUUIDFromBytes call took " + total + " ms");
		
		// test creating UUIDs from a String
		start = System.currentTimeMillis();
		for (int i = 0; i < NUM_ITS; i++) {
			uuid = UUID.fromString(uuid.toString());
		}
		end = System.currentTimeMillis();
		total = end - start;
		LOG.info("Creating " + NUM_ITS + " UUIDs via the fromString() call took " + total + " ms");
		
		// finally, test generating random UUIDs
		start = System.currentTimeMillis();
		for (int i = 0; i < NUM_ITS; i++) {
			uuid = UUID.randomUUID();
		}
		end = System.currentTimeMillis();
		total = end - start;
		LOG.info("Creating " + NUM_ITS + " UUIDs via the randomUUID call took " + total + " ms");
		
	}
	
}
